import { __awaiter } from 'tslib';
import { Component, HostListener, ChangeDetectorRef, Input, Output, EventEmitter, forwardRef, ViewEncapsulation, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { PickerController } from '@ionic/angular';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const CITY_PICKER_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef((/**
     * @return {?}
     */
    () => IonCityPicker)),
    multi: true
};
class IonCityPicker {
    /**
     * @param {?} pickerCtrl
     * @param {?} changeDetectorRef
     */
    constructor(pickerCtrl, changeDetectorRef) {
        this.pickerCtrl = pickerCtrl;
        this.changeDetectorRef = changeDetectorRef;
        this.cityIds = 0;
        this.inputId = `ion-city-${this.cityIds++}`;
        this.labelId = this.inputId + `-lbl`;
        this.cityValue = {};
        this.cityText = '';
        this.addPlaceholderClass = false;
        this.isExpanded = false;
        this.isLoadData = false;
        this.provinceCol = 0;
        this.cityCol = 0;
        this.regionCol = 0;
        this.onChange = (/**
         * @param {?} randomNumber
         * @return {?}
         */
        (randomNumber) => { });
        this.onTouch = (/**
         * @return {?}
         */
        () => { });
        /**
         * The value of the city string.
         */
        this.value = '';
        /**
         * The name of the control, which is submitted with the form data.
         */
        this.name = this.inputId;
        /**
         * If `true`, the user cannot interact with the city.
         */
        this.disabled = false;
        /**
         * If `true`, the city appears normal but is not interactive.
         */
        this.readonly = false;
        /**
         * \@input {string} The text to display on the picker's cancel button. Default: `Cancel`.
         */
        this.cancelText = 'Cancel';
        /**
         * \@input {string} The text to display on the picker's "Done" button. Default: `Done`.
         */
        this.doneText = 'Done';
        /**
         * \@input {CityPickerColumn} city data
         */
        this.citiesData = [];
        /**
         * \@input {string} separate
         */
        this.separator = '-';
        /**
         * \@output {any} Emitted when the city selection has changed.
         */
        this.ionChange = new EventEmitter();
        /**
         * \@output {any} Emitted when the city selection was cancelled.
         */
        this.ionCancel = new EventEmitter();
        /**
         * Emitted when the styles change.
         * \@internal
         */
        this.ionStyle = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this.cityText = this.value;
        if (this.cityText === '') {
            this.cityText = this.placeholder != null ? this.placeholder : '';
        }
        this.hasReadonly();
        this.hasDisable();
        this.updateCityValue(this.value);
        this.emitStyle();
    }
    /**
     * @return {?}
     */
    ngAfterContentChecked() {
        if (this.isLoadData || this.citiesData.length === 0) {
            return;
        }
        this.hasPlaceholder();
        this.updateCityValue(this.value);
        this.isLoadData = true;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    writeValue(val) {
        this.setValue(val);
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouch = fn;
    }
    /**
     * @param {?} ev
     * @return {?}
     */
    onClick(ev) {
        if (ev.detail === 0) {
            // do not continue if the click event came from a form submit
            return;
        }
        this.setFocus();
        this.open();
    }
    /**
     * @private
     * @return {?}
     */
    open() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.disabled || this.isExpanded) {
                return;
            }
            /** @type {?} */
            const pickerOptions = this.generatePickerOptions();
            /** @type {?} */
            const picker = yield this.pickerCtrl.create(pickerOptions);
            this.isExpanded = true;
            picker.onDidDismiss().then((/**
             * @return {?}
             */
            () => {
                this.isExpanded = false;
                this.setFocus();
            }));
            picker.addEventListener('ionPickerColChange', (/**
             * @param {?} event
             * @return {?}
             */
            (event) => __awaiter(this, void 0, void 0, function* () {
                /** @type {?} */
                const data = event.detail;
                if (data.name !== 'province' && data.name !== 'city' && data.name !== 'region') {
                    return;
                }
                /** @type {?} */
                const colSelectedIndex = data.selectedIndex;
                /** @type {?} */
                const colOptions = data.options;
                /** @type {?} */
                const changeData = {};
                changeData[data.name] = {
                    value: colOptions[colSelectedIndex].value
                };
                this.updateCityValue(changeData);
                /** @type {?} */
                const columns = this.generateColumns();
                picker.columns = columns;
                yield this.valiCity(picker);
            })));
            yield this.valiCity(picker);
            yield picker.present();
        });
    }
    /**
     * @private
     * @return {?}
     */
    emitStyle() {
        this.hasPlaceholder();
        this.ionStyle.emit({
            'interactive': true,
            'city': true,
            'has-placeholder': this.placeholder != null,
            'has-value': this.hasValue(),
            'interactive-disabled': this.disabled,
        });
    }
    /**
     * @private
     * @param {?} value
     * @return {?}
     */
    updateCityValue(value) {
        return __awaiter(this, void 0, void 0, function* () {
            this.updateValue(this.cityValue, value, this.separator);
        });
    }
    /**
     * @private
     * @return {?}
     */
    generatePickerOptions() {
        /** @type {?} */
        const pickerOptions = Object.assign({ mode: this.mode }, this.pickerOptions, { columns: this.generateColumns() });
        // If the user has not passed in picker buttons,
        // add a cancel and ok button to the picker
        /** @type {?} */
        const buttons = pickerOptions.buttons;
        if (!buttons || buttons.length === 0) {
            pickerOptions.buttons = [
                {
                    text: this.cancelText,
                    handler: (/**
                     * @param {?} data
                     * @return {?}
                     */
                    (data) => {
                        this.ionCancel.emit(null);
                    })
                }, {
                    text: this.doneText,
                    handler: (/**
                     * @param {?} data
                     * @return {?}
                     */
                    (data) => {
                        this.value = this.convertDuration(data, 'value');
                        this.cityText = this.convertDuration(data, 'text');
                        this.updateCityValue(data);
                        this.emitStyle();
                        this.ionChange.emit(this);
                        this.changeDetectorRef.detectChanges();
                    })
                }
            ];
        }
        return pickerOptions;
    }
    /**
     * @private
     * @param {?} durationObject
     * @param {?} type
     * @return {?}
     */
    convertDuration(durationObject, type) {
        /** @type {?} */
        const province = durationObject.province[type].toString();
        /** @type {?} */
        const city = durationObject.city[type].toString();
        /** @type {?} */
        const region = durationObject.region[type].toString();
        return province + this.separator + city + this.separator + region;
    }
    /**
     * @private
     * @return {?}
     */
    generateColumns() {
        /** @type {?} */
        const columns = [];
        // const values = this.value.toString().split(this.separator);
        // Add province data to picker
        /** @type {?} */
        const provinceCol = {
            name: 'province',
            options: this.citiesData.map((/**
             * @param {?} province
             * @return {?}
             */
            province => ({ text: province.name, value: province.code, disabled: false, duration: 100 })))
        };
        /** @type {?} */
        let provinceIndex = this.citiesData.findIndex((/**
         * @param {?} option
         * @return {?}
         */
        option => option.code === this.cityValue.province));
        provinceCol.selectedIndex = provinceIndex === -1 ? 0 : provinceIndex;
        columns.push(provinceCol);
        // Add city data to picker
        /** @type {?} */
        const cityColData = this.citiesData[provinceCol.selectedIndex].children;
        /** @type {?} */
        const cityCol = {
            name: 'city',
            options: cityColData.map((/**
             * @param {?} city
             * @return {?}
             */
            city => ({ text: city.name, value: city.code, disabled: false, duration: 100 })))
        };
        /** @type {?} */
        let cityIndex = cityColData.findIndex((/**
         * @param {?} option
         * @return {?}
         */
        option => option.code === this.cityValue.city));
        cityCol.selectedIndex = cityIndex === -1 ? 0 : cityIndex;
        columns.push(cityCol);
        // Add region data to picker
        /** @type {?} */
        const regionData = this.citiesData[provinceCol.selectedIndex].children[cityCol.selectedIndex].children;
        /** @type {?} */
        const regionColCol = {
            name: 'region',
            options: regionData.map((/**
             * @param {?} city
             * @return {?}
             */
            city => ({ text: city.name, value: city.code, disabled: false, duration: 100 })))
        };
        /** @type {?} */
        let regionIndex = regionData.findIndex((/**
         * @param {?} option
         * @return {?}
         */
        option => option.code === this.cityValue.region));
        regionColCol.selectedIndex = regionIndex === -1 ? 0 : regionIndex;
        columns.push(regionColCol);
        this.provinceCol = provinceCol.selectedIndex;
        this.cityCol = cityCol.selectedIndex;
        this.regionCol = regionColCol.selectedIndex;
        return this.divyColumns(columns);
    }
    /**
     * @private
     * @param {?} picker
     * @return {?}
     */
    valiCity(picker) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            let columns = picker.columns;
            /** @type {?} */
            let provinceCol = columns[0];
            /** @type {?} */
            let cityCol = columns[1];
            /** @type {?} */
            let regionCol = columns[2];
            if (cityCol && this.provinceCol != provinceCol.selectedIndex) {
                cityCol.selectedIndex = 0;
                /** @type {?} */
                let cityColData = this.citiesData[provinceCol.selectedIndex].children;
                cityCol.options = cityColData.map((/**
                 * @param {?} city
                 * @return {?}
                 */
                city => { return { text: city.name, value: city.code, disabled: false }; }));
            }
            if (regionCol && (this.cityCol != cityCol.selectedIndex || this.provinceCol != provinceCol.selectedIndex)) {
                /** @type {?} */
                let regionData = this.citiesData[provinceCol.selectedIndex].children[cityCol.selectedIndex].children;
                regionCol.selectedIndex = 0;
                regionCol.options = regionData.map((/**
                 * @param {?} city
                 * @return {?}
                 */
                city => { return { text: city.name, value: city.code, disabled: false }; }));
            }
            this.provinceCol = provinceCol.selectedIndex;
            this.cityCol = cityCol.selectedIndex;
            this.regionCol = regionCol.selectedIndex;
        });
    }
    /**
     * @param {?} newData
     * @return {?}
     */
    setValue(newData) {
        if (newData === null || newData === undefined) {
            this.value = '';
        }
        else {
            this.value = newData;
        }
        this.updateCityValue(this.value);
        this.getText();
    }
    /**
     * @return {?}
     */
    getValue() {
        return this.value;
    }
    /**
     * @return {?}
     */
    getText() {
        if (this.value === undefined ||
            this.value === null ||
            this.value.length === 0) {
            return;
        }
        this.cityText = this.renderValue(this.value);
        return this.cityText;
    }
    /**
     * @private
     * @return {?}
     */
    hasValue() {
        /** @type {?} */
        const val = this.cityValue;
        return Object.keys(val).length > 0;
    }
    /**
     * @private
     * @return {?}
     */
    setFocus() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.buttonEl) {
                this.buttonEl.focus();
            }
        });
    }
    /**
     * @private
     * @param {?} columns
     * @return {?}
     */
    divyColumns(columns) {
        /** @type {?} */
        const columnsWidth = [];
        /** @type {?} */
        let col;
        /** @type {?} */
        let width;
        for (let i = 0; i < columns.length; i++) {
            col = columns[i];
            columnsWidth.push(0);
            for (const option of col.options) {
                width = (/** @type {?} */ (option.text)).length;
                if (width > columnsWidth[i]) {
                    columnsWidth[i] = width;
                }
            }
        }
        if (columnsWidth.length === 2) {
            width = Math.max(columnsWidth[0], columnsWidth[1]);
            columns[0].align = 'right';
            columns[1].align = 'left';
            columns[0].optionsWidth = columns[1].optionsWidth = `${width * 17}px`;
        }
        else if (columnsWidth.length === 3) {
            width = Math.max(columnsWidth[0], columnsWidth[2]);
            columns[0].align = 'right';
            columns[2].align = 'left';
        }
        return columns;
    }
    /**
     * @private
     * @param {?} existingData
     * @param {?} newData
     * @param {?} separator
     * @return {?}
     */
    updateValue(existingData, newData, separator) {
        if (newData && newData !== '') {
            if (typeof newData === 'string') {
                // new value is a string
                // convert it to our CityData if a valid
                newData = this.parseValue(newData);
                if (newData) {
                    // successfully parsed string to our CityData
                    Object.assign(existingData, newData);
                    return true;
                }
            }
            else if ((newData.province || newData.city || newData.region)) {
                // merge new values from the picker's selection
                // to the existing CityData values
                for (const key of Object.keys(newData)) {
                    ((/** @type {?} */ (existingData)))[key] = newData[key].value;
                }
                return true;
            }
        }
        else {
            // blank data, clear everything out
            for (const k in existingData) {
                if (existingData.hasOwnProperty(k)) {
                    delete ((/** @type {?} */ (existingData)))[k];
                }
            }
        }
        return false;
    }
    /**
     * @private
     * @param {?} val
     * @return {?}
     */
    parseValue(val) {
        /** @type {?} */
        let parse = null;
        if (val != null && val !== '') {
            // try parsing for just value first
            parse = val.split(this.separator);
        }
        if (parse === null) {
            // wasn't able to parse the value
            return undefined;
        }
        return {
            province: parse[0],
            city: parse[1],
            region: parse[2]
        };
    }
    /**
     * @private
     * @param {?} val
     * @return {?}
     */
    renderValue(val) {
        /** @type {?} */
        let parse = null;
        if (val != null && val !== '') {
            // try parsing for just value first
            parse = val.split(this.separator);
        }
        if (parse === null) {
            // wasn't able to parse the value
            return undefined;
        }
        /** @type {?} */
        let textValue;
        this.citiesData.forEach((/**
         * @param {?} pro
         * @return {?}
         */
        pro => {
            if (pro.code !== parse[0]) {
                return;
            }
            textValue = pro.name;
            pro.children.forEach((/**
             * @param {?} city
             * @return {?}
             */
            city => {
                if (city.code !== parse[1]) {
                    return;
                }
                textValue = textValue + this.separator + city.name;
                city.children.forEach((/**
                 * @param {?} reg
                 * @return {?}
                 */
                reg => {
                    if (reg.code !== parse[2]) {
                        return;
                    }
                    textValue = textValue + this.separator + reg.name;
                }));
            }));
        }));
        return textValue;
    }
    /**
     * @private
     * @return {?}
     */
    hasPlaceholder() {
        return __awaiter(this, void 0, void 0, function* () {
            this.addPlaceholderClass =
                (this.getText() === undefined && this.placeholder != null) ? true : false;
        });
    }
    /**
     * @private
     * @return {?}
     */
    hasReadonly() {
        return __awaiter(this, void 0, void 0, function* () {
            this.readonly =
                (this.readonly.toString() === '' || this.readonly.toString() === 'true' || this.readonly === true) ? true : false;
        });
    }
    /**
     * @private
     * @return {?}
     */
    hasDisable() {
        return __awaiter(this, void 0, void 0, function* () {
            this.disabled =
                (this.disabled.toString() === '' || this.disabled.toString() === 'true' || this.disabled === true) ? true : false;
        });
    }
}
IonCityPicker.decorators = [
    { type: Component, args: [{
                selector: 'ionic4-city-picker',
                template: '<div class="city-picker-text">{{cityText}}</div>' +
                    '<button type="button"  ' +
                    'role="ombobox" ' +
                    'aria-haspopup="true"  ' +
                    '[attr.aria-labelledby]="labelId" ' +
                    '[attr.aria-disabled]="disabled" ' +
                    '[attr.aria-expanded]="isExpanded" ' +
                    'class="city-picker-button">' +
                    '</button>',
                host: {
                    '[class.city-picker-disabled]': 'disabled',
                    '[class.city-picker-readonly]': 'readonly',
                    '[class.city-picker-placeholder]': 'addPlaceholderClass'
                },
                providers: [CITY_PICKER_VALUE_ACCESSOR],
                encapsulation: ViewEncapsulation.None,
                styles: ["ion-city-picker{padding-top:10px;padding-bottom:10px;display:flex;overflow:hidden}.city-picker-button{left:0;top:0;position:absolute;width:100%;height:100%;border:0;background:0 0;cursor:pointer;-webkit-appearance:none;-moz-appearance:none;appearance:none;outline:0;margin:0}.city-picker-text{margin-right:8px;font-family:inherit;font-size:inherit;font-style:inherit;font-weight:inherit;letter-spacing:inherit;text-decoration:inherit;text-overflow:inherit;text-transform:inherit;text-align:inherit;white-space:inherit;color:inherit;flex:1;min-height:inherit;direction:ltr;overflow:inherit}.item-label-floating city-picker,.item-label-stacked city-picker{padding-left:0;width:100%}.picker-opt{margin-left:4px;font-size:18px}.picker-opts-left .picker-opts .picker-opt{margin-left:-8px}.picker-opts-right .picker-opts .picker-opt{margin-left:15px}.in-item{position:static}.city-picker-placeholder{color:#999}.city-picker-disabled,.item-city-picker-disabled ion-label{opacity:.3;pointer-events:none}.city-picker-readonly{pointer-events:none}"]
            }] }
];
/** @nocollapse */
IonCityPicker.ctorParameters = () => [
    { type: PickerController },
    { type: ChangeDetectorRef }
];
IonCityPicker.propDecorators = {
    value: [{ type: Input }],
    mode: [{ type: Input }],
    name: [{ type: Input }],
    disabled: [{ type: Input }],
    readonly: [{ type: Input }],
    pickerOptions: [{ type: Input }],
    cancelText: [{ type: Input }],
    doneText: [{ type: Input }],
    citiesData: [{ type: Input }],
    separator: [{ type: Input }],
    placeholder: [{ type: Input }],
    ionChange: [{ type: Output }],
    ionCancel: [{ type: Output }],
    ionStyle: [{ type: Output }],
    onClick: [{ type: HostListener, args: ['click', ['$event.target'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class IonCityPickerModule {
}
IonCityPickerModule.decorators = [
    { type: NgModule, args: [{
                declarations: [IonCityPicker],
                imports: [],
                exports: [IonCityPicker]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { IonCityPickerModule, CITY_PICKER_VALUE_ACCESSOR, IonCityPicker };

//# sourceMappingURL=ionic4-city-picker.js.map